<?php
namespace Teknisa\CMP\Modules\V1\Controllers; 

use Teknisa\GER\Modules\V1\Services\NovoCodigo;
use \Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\CMP\Modules\V1\Services\SolicitacaoComprasSave;
use Zeedhi\Framework\Log\Exception;

class SolicitacaoComprasJustificativaController extends \Teknisa\GER\Modules\V1\Controllers\TeknisaCrud {

    protected $dataSourceName = "solicitacao_compras_justificativa";
    protected $dataSourceNameToSaveItemsoli = "solicitacao_compras_itemsoli_save";
    protected $solicitacaoComprasSave;
    protected $novoCodigo;

    protected $paramterBagColumns = array(
        "DTPROGENTR"
    );

    public function __construct(Manager $dataSourceManager, Environment $environment, ParameterBag $parameterBag,
                                NovoCodigo $novoCodigo, SolicitacaoComprasSave $solicitacaoComprasSave)
    {
        parent::__construct($dataSourceManager, $environment, $parameterBag);
        $this->novoCodigo = $novoCodigo;
        $this->solicitacaoComprasSave = $solicitacaoComprasSave;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        $this->parameterBag->set('CDFILIAL', $this->environment->getCdFilial());
        $this->parameterBag->set('NRORG', $this->environment->getNrOrg());
        parent::find($request, $response);
    }

    public function save(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $tipoSol = $row->get("IDTIPLAN");

        if (strtoupper($tipoSol) == "MANUAL EXTRA" || strtoupper($tipoSol) == "EXTRA") {
            $idSolExtra = "S";
            $idTipLan = "M";
        } elseif (strtoupper($tipoSol) == "MANUAL" || strtoupper($tipoSol) == "NORMAL") {
            $idSolExtra = "N";
            $idTipLan = "M";
        } else {
            $idSolExtra = "N";
            $idTipLan = "A";
        }
        if (strlen($row->get("NRSOLICMP")) == 0) {
            $isNew = true;
        } else {
            $isNew = false;
        }
        $rowToSave = array(
            "CDPRODUTO" => $row->get("CDPRODUTO"),
            "CDFILLAN" => $row->get("CDFILLAN"),
            "CDFILSOLI" => $row->get("CDFILSOLI"),
            "CDOPERLANC" => $row->get("CDOPERLANC"),
            "DTPROGENTR" => $row->get("DTPROGENTR"),
            "QTSOLIPROD" => $row->get("QTSOLIPROD"),
            "DSJUSTOPER" => $row->get("DSJUSTOPER"),
            "NRSOLICMP" => $row->get("NRSOLICMP"),
            "DTUTILIZA" => $row->get("DTUTILIZA"),
            "CDDESTSOLI" => $row->get("CDDESTSOLI"),
            "CDCOMPRADOR" => $row->get("CDCOMPRADOR"),
            "IDSOLEXTRA" => $idSolExtra,
            "IDTIPLAN" => $idTipLan,
            "NRORG" => $this->environment->getNrOrg(),
            "CDOPERADOR" => $this->environment->getCdOperador(),
            "DTPROGENTR_old" => $row->get("DTPROGENTR_old"),
            "CDDESTSOLI_old" => $row->get("CDDESTSOLI_old"),
            "CDCOMPRADOR_old" => $row->get("CDCOMPRADOR_old"),
            "__is_new" => $isNew
        );
        $bdRows = $this->solicitacaoComprasSave->hasItemSoli($rowToSave);
        if (empty($bdRows)) {
            $response->addMessage(new Message($this->insertItem($rowToSave)));
        } else {
            $response->addMessage(new Message($this->updateItem($bdRows[0], $rowToSave)));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response)
    {
        $request = $this->buildDataSetIfRow($request);
        $rowToDelete = $request->getDataSet()->getRows()[0];
        $rowToDelete['NRORG'] = $this->environment->getNrOrg();
        $this->solicitacaoComprasSave->deleteItemSoli($rowToDelete);
        $response->addMessage(new Message("Row deleted with success"));
    }

    public function updateItem($bdRow, $rowToUpdate)
    {
        $rowToUpdate['DTATU'] = $this->environment->getCurrentTime()->format("d/m/Y");

        if ($rowToUpdate["__is_new"] == false &&
            ($rowToUpdate['DTPROGENTR'] != $rowToUpdate['DTPROGENTR_old'] ||
                $rowToUpdate['CDDESTSOLI'] != $rowToUpdate['CDDESTSOLI_old'] ||
                $rowToUpdate['CDCOMPRADOR'] != $rowToUpdate['CDCOMPRADOR_old'])) {
            try {
                $this->solicitacaoComprasSave->deleteItemSoli($rowToUpdate);
            } catch (Exception $e) {
                return "The old row couldn't be deleted. \n" + $e;
            }
        }
        try {
            $rowToUpdate['NRSOLICMP'] = $bdRow['NRSOLICMP'];
            $this->solicitacaoComprasSave->updateItemSoli($rowToUpdate);
            $this->solicitacaoComprasSave->insertMovoPsol($rowToUpdate);
            return "1 row actualised with success";
        } catch (Exception $e) {
            return "This row couldn't be actualised. \n" + $e;
        }

    }

    public function insertItem($rowToInsert)
    {
        $rowToInsert['DTSOLCMP'] = $this->environment->getCurrentTime()->format("d/m/Y");
        $bdRow = $this->solicitacaoComprasSave->hasSolicita($rowToInsert);

        if ($rowToInsert["__is_new"] == false) {
            try {
                $this->solicitacaoComprasSave->deleteItemSoli($rowToInsert);
            } catch (Exception $e) {
                return "The old row couldn't be deleted. \n" + $e;
            }
        }

        if (empty($bdRow)) {
            $tablename = 'SOLICOMP' . $rowToInsert["CDFILLAN"] . $rowToInsert["CDFILSOLI"];
            try {

                $rowToInsert['NRSOLICMP'] = $this->novoCodigo->getCodeByTableName($tablename, $this->environment->getNrOrg());
                $this->solicitacaoComprasSave->insertSolicita($rowToInsert);
            } catch (Exception $e) {
                return "Solicita new row couldn't be inserted. \n" + $e;
            }
        }
        else {
            $rowToInsert['NRSOLICMP'] = $bdRow[0]['NRSOLICMP'];
        }

        try {
            $this->solicitacaoComprasSave->insertItemSoli($rowToInsert);
            $this->solicitacaoComprasSave->insertMovoPsol($rowToInsert);
            return "Rows in Solicita, ItemSoli, MovoPsol inserted with success";
        } catch (Exception $e) {
            return "ItemSoli or/and MovoPsol new row couldn't be inserted. \n" + $e;
        }
    }
}